WMarkDown.Dictionary = function(wmarkdown){

    const self = this;
    let started = false,
        scroll_event = null,
        z_index = 5,
        thread = null,
        cache = "";

    const construct = () => {};

    const scroll_event_function = event => {

        const content_box = wmarkdown.item_self.querySelector(".content"),
              half_width = content_box.offsetWidth / 2,
              half_height = content_box.offsetHeight / 2,
              font_size = window.getComputedStyle(wmarkdown.item_self, null).getPropertyValue("font-size");

        content_box.querySelectorAll(".content .data-dictionary-item").forEach(item => {

            const properties = item.getBoundingClientRect();

            item.setAttribute("data-left", properties.x - content_box.offsetLeft > half_width);
            item.setAttribute("data-top", properties.y - content_box.offsetTop < half_height);
            item.querySelector(".box").style.fontSize = font_size;

        });

    };

    const autosearch = () => wmarkdown.item_self.querySelectorAll(".data-dictionary-table").forEach(searcher => searcher.querySelector("[name=autosearch]").checked && self.search(searcher, null));

    this.start = () => {

        if(started)
            return;
        started = true;

        const dictionary_links = wmarkdown.settings("dictionary_links");
              dictionary_title = wmarkdown.settings("dictionary_title") || "Dictionary";

        scroll_event = wmarkdown.item_self.querySelector(".content").addEventListener("scroll", scroll_event_function);
        self.build();
        thread = wmarkdown.threads_add(autosearch);

        if(dictionary_links)
            load_links(dictionary_links.push ? dictionary_links : [dictionary_links], dictionary => {

                const content_box = wmarkdown.item_self.querySelector(".content-box");
                let hash = dictionary_title.toLowerCase().replace(/[^a-z\d]+/g, "-");

                if(!/^(.+\-)(\d+)$/.test(hash))
                    hash += "-0";
                while(document.getElementById(hash))
                    hash = hash.replace(/^(.+\-)(\d+)$/, (...arguments) => arguments[1] + (Number(arguments[2]) + 1));

                with(content_box.appendChild(document.createElement("h1"))){
                    setAttribute("id", hash);
                    innerText = dictionary_title;
                };
                with(wmarkdown.item_self.querySelector(".headers-menu>nav>ul").appendChild(document.createElement("li"))){
                    setAttribute("data-level", 1);
                    setAttribute("data-deployed", false);
                    setAttribute("data-levels", 0);
                    setAttribute("data-parent-deployed", true);
                    setAttribute("data-id", wmarkdown.hash());
                    innerHTML = (`
                        <span data-icon="deploy" onlick="` + wmarkdown.object_name + `.deploy(this, event);"></span>
                        <a href="#` + hash + `" title="` + dictionary_title + `" target="_self" onclick="` + wmarkdown.object_name + `.deploy(this, event);` + wmarkdown.object_name + `.go_to_item(this, event);">` + dictionary_title + `</a>
                    `);
                };
                build(dictionary, content_box.appendChild(document.createElement("form")));

            });

    };

    const process_description = description => {
        if(!description)
            return "";
        if(description.substr)
            return description;
        if(typeof description != "object")
            return "" + description;

        let html = `<span class="paragraph">`;

        description.forEach(line => html += " " + (line || `</span><span class="paragraph">`));
        html += `</span>`;

        return html;
    }

    const mount = (dictionary, item) => item.childNodes.forEach(node => {
        if(node.hasAttribute && node.hasAttribute("data-ignore") && node.getAttribute("data-ignore") == "true")
            return;
        if(node.nodeType != Node.TEXT_NODE){
            !(node.tagName && ["noscript", "a"].includes(node.tagName.toLowerCase())) && !(node.classList && node.classList.contains("code-block")) && mount(dictionary, node);
            return;
        };

        let html = node.textContent;

        if(!node.textContent || !/[^\r\n\s\t]/.test(html))
            return;

        const nodes = [];

        dictionary.forEach(item => item.patterns.forEach((pattern, i) => pattern && (html = html.replace(pattern, (...values) => {

            let html = ``;
            const result = (item.results[i] || item.results[0]).replace(/\$(\d)/g, (all, i) => values[Number(i)] || "");

            (item.links ? item.links.push ? item.links : [item.links] : []).forEach(link => html += `<a href="` + link + `" title="` + link + `" target="_blank" data-site="` + link.replace(/^[^\:]+\:\/{2}([^\/]+)(\/(.+)?)?$/, "$1").replace(/[^a-z\d]+/g, "_") + `"></a>`);
            nodes.push((`
                <span
                    class="data-dictionary-item"
                    onmouseover="` + wmarkdown.object_name + `.dictionary.show(this, event);"
                    onmouseout="` + wmarkdown.object_name + `.dictionary.hide(this, event);"
                    data-show="false"
                ><span class="box">
                    <span class="result">` + result + `</span>
                    <span class="description">` + process_description(item.description) + `</span>
                    <span class="links">` + html + `</span>
                </span><span class="text">` + result + `</span></span>
            `).trim());

            return '###' + (nodes.length - 1) + "###";
        }))));

        html = html.replace(/\#{3}(\d+)\#{3}/g, (...arguments) => nodes[Number(arguments[1])]);

        if(html == node.textContent)
            return;

        const span = node.parentNode.insertBefore(document.createElement("span"), node);

        span.setAttribute("class", "data-dictionary-block");
        span.innerHTML = html;

        node.remove();

    });

    const load = (files, i, callback, dictionary, patterns_used) => {

        if(i >= files.length){
            callback(dictionary);
            return;
        };

        wmarkdown.load_file(files[i], data => {
            if(data){
                try{
                    JSON.parse(data).forEach(item => {

                        const json = item.pattern.substr ? item.pattern : JSON.stringify(item.pattern);

                        if(patterns_used.includes(json))
                            return;
                        patterns_used.push(json);

                        let pattern;
                        const patterns = [];

                        if(!item.pattern)
                            return;

                        (item.pattern.push ? item.pattern : [item.pattern]).forEach(pattern => {

                            const pattern_closed = pattern.lastIndexOf("/");

                            try{
                                pattern_closed && (pattern = new RegExp(pattern.substr(1, pattern_closed - 1), pattern.substr(pattern_closed + 1)));
                            }catch(exception){};

                            patterns.push(pattern || null);

                        });

                        patterns && dictionary.push(Object.assign({}, item, {
                            patterns : patterns,
                            results : item.result.push ? item.result : [item.result]
                        }));

                    });
                }catch(exception){console.error(exception);};
            };
            load(files, i + 1, callback, dictionary, patterns_used);
        });

    };

    const build = (dictionary, item) => {

        const header = (`
                  <tr>
                      <th data-field="i" data-i18n="i">i</th>
                      <th data-field="patterns" data-i18n="patterns">Patterns</th>
                      <th data-field="results" data-i18n="results">Results</th>
                      <th data-field="words" data-i18n="words">Words</th>
                      <th data-field="meanings" data-i18n="meanings">Meanings</th>
                      <th data-field="urls" data-i18n="urls">URLs</th>
                  </tr>
              `),
              hashes = {};
        let body = ``;

        ["search", "autosearch", "meanings", "case_sensitive"].forEach(key => hashes[key] = wmarkdown.hash());

        dictionary.forEach((item, i) => {

            let urls = ``,
                words = '';
            const results = item.words || item.results;

            (item.links ? item.links.push ? item.links : [item.links] : []).forEach(link => urls += `<li><a href="` + link + `" target="_blank" title="` + link + `">` + link + `</a></li>`);
            (results.push ? results : [results]).forEach(word => words += '<li>' + word + '</li>');

            body += (`
                <tr data-i="` + i + `" data-visible="true">
                    <td data-field="i">` + i + `</td>
                    <td data-field="patterns"><ul><li>` + item.patterns.join(`</li><li>`) + `</li></ul></td>
                    <td data-field="results"><ul><li>` + item.results.join(`</li><li>`) + `</li></ul></td>
                    <td data-field="words"><ul>` + words + `</ul></td>
                    <td data-field="description">` + (item.description.push ? item.description.join(" ") : item.description) + `</td>
                    <td data-field="urls"><ul>` + urls + `</ul></td>
                </tr>
            `);
        });

        item.setAttribute("class", "data-dictionary-table");
        item.setAttribute("action", window.location.hash);
        item.setAttribute("method", "get");
        item.setAttribute("onsubmit", "return false;");
        item.innerHTML = (`
            <fieldset>
                <legend data-i18n="search">Search</legend>
                <div class="search" data-i18n="search" data-i18n-without="true" title="search">
                    <label for="` + hashes.search + `" data-i18n="search">Search</label>
                    <span class="field"><input type="text" id="` + hashes.search + `" name="search" data-i18n="search" data-i18n-without="true" placeholder="Search..." /></span>
                    <span class="items">
                        <label class="checkbox" for="` + hashes.meanings + `" data-i18n="meanings" data-i18n-without="true" title="Meanings">
                            <input type="checkbox" id="` + hashes.meanings + `" name="meanings"` + (wmarkdown.settings("meanings") ? `checked` : ``) + ` />
                            <span data-icon="checkbox"></span>
                            <span data-i18n="meanings">Meanings</span>
                        </label>
                        <label class="checkbox" for="` + hashes.case_sensitive + `" data-i18n="case_sensitive" data-i18n-without="true" title="Case sensitive">
                            <input type="checkbox" id="` + hashes.case_sensitive + `" name="case_sensitive"` + (wmarkdown.settings("case_sensitive") ? `checked` : ``) + ` />
                            <span data-icon="checkbox"></span>
                            <span data-i18n="case_sensitive">Case sensitive</span>
                        </label>
                        <label class="checkbox" for="` + hashes.autosearch + `" data-i18n="autosearch" data-i18n-without="true" title="Autosearch">
                            <input type="checkbox" id="` + hashes.autosearch + `" name="autosearch"` + (wmarkdown.settings("autosearch") ? `checked` : ``) + ` />
                            <span data-icon="checkbox"></span>
                            <span data-i18n="autosearch">Autosearch</span>
                        </label>
                        <button type="button" data-i18n="search" data-without="true" title="Search" onclick="` + wmarkdown.object_name + `.dictionary.search(this, event);">
                            <span data-icon="search"></span>
                            <span data-i18n="search">Search</span>
                        </button>
                    </span>
                </div>
                <div class="table">
                    <table>
                        <thead>` + header + `</thead>
                        <tbody>` + body + `</tbody>
                        <tfoot>` + header + `</tfoot>
                    </table>
                </div>
            </fieldset>
        `);
    }

    const load_links = (links, callback) => load(links, 0, dictionary => {

        mount(dictionary, wmarkdown.item_self.querySelector(".content>.content-box"));
        setTimeout(scroll_event_function, 10);

        typeof callback == "function" && callback(dictionary);

    }, [], []);

    this.build = () => wmarkdown.item_self.querySelectorAll(".wdictionary[data-status=unloaded]").forEach(items => {
        items.setAttribute("data-status", "loading");

        load_links(JSON.parse(wmarkdown.utf8_decode(atob(items.getAttribute("data-json")))), dictionary => {
            build(dictionary, items.parentNode.insertBefore(document.createElement("form"), items));
        });

    });

    this.show = (element, event) => {
        element.style.zIndex = ++ z_index;
        element.setAttribute("data-show", true);
    };
    this.hide = (element, event) => element.setAttribute("data-show", false);

    this.search = (element, event) => {
        if(!element)
            return;

        while((!element.classList || !element.classList.contains("data-dictionary-table")) && (element = element.parentNode));

        let text = element.querySelector("[name=search]").value;

        if(text == cache)
            return;
        cache = text;

        const meanings = element.querySelector("[name=meanings]").checked,
              case_sensitive = element.querySelector("[name=case_sensitive]").checked;

        !case_sensitive && (text = text.toLowerCase());

        element.querySelectorAll("tbody>tr").forEach(word => {

            let words = word.querySelector("[data-field=words]").innerText,
                description = word.querySelector("[data-field=description]").innerText;

            if(!case_sensitive){
                words = words.toLowerCase(),
                description = description.toLowerCase();
            };

            word.setAttribute("data-visible", (
                !text ||
                words.indexOf(text) != -1 ||
                (meanings && description.indexOf(text) != -1)
            ));

        });

    };

    construct();

};
